import { createRouter, createWebHashHistory } from 'vue-router';
import layout from '@/layout/index.vue';
import { useUserStore } from '@/store/user.js';


const routes = [
  {
    path: '/',
    name: 'layout',
    component: layout,
    redirect: '/saleHome',
    meta: { title: '里海收银系统' },
    children: [
      {
        path: '/saleHome',
        name: 'saleHome',
        meta: { title: '收银' },
        component: () => import('@/views/saleHome/index.vue'),
      },
      {
        path: '/saleOrder',
        name: 'saleOrder',
        meta: { title: '收银订单' },
        component: () => import('@/views/saleOrder/index.vue'),
      },
      {
        path: '/order',
        name: 'order',
        meta: { title: '线上采购' },
        component: () => import('@/views/order/index.vue'),
      },
      {
        path: '/subOrder',
        name: 'subOrder',
        meta: { title: '订单统计' },
        component: () => import('@/views/subOrder/index.vue'),
      },
      {
        path: '/order',
        name: 'order',
        meta: { title: '预约取货' },
        component: () => import('@/views/order/index.vue'),
      },
      {
        path: '/wallet',
        name: 'wallet',
        meta: { title: '提现' },
        component: () => import('@/views/wallet/index.vue'),
      },
      {
        path: '/convert',
        name: 'convert',
        meta: { title: '订单核销' },
        component: () => import('@/views/convert/index.vue'),
      },
      {
        path: '/putStorage',
        name: 'putStorage',
        meta: { title: '入库登记' },
        component: () => import('@/views/putStorage/index.vue'),
      },
      {
        path: '/stock',
        name: 'stock',
        meta: { title: '入库登记' },
        component: () => import('@/views/stock/index.vue'),
      },
      {
        path: '/user',
        name: 'user',
        meta: { title: '用户' },
        component: () => import('@/views/user/index.vue'),
      },
      {
        path: '/test',
        name: 'test',
        component: () => import('@/views/test/test.vue'),
      },
    ]
  },
  {
    path: '/login',
    name: 'login',
    component: () => import('@/views/login/index.vue'),
  },
  // 注意：通配符路由要放在最后
  {
    path: '/:catchAll(.*)',
    name: 'notFound',
    component: () => import('@/views/notFound/index.vue') // 使用 404 页面组件
  }
];

const router = createRouter({
  history: createWebHashHistory(),
  routes
});

// 全局前置守卫
router.beforeEach((to, from, next) => {
  const userStore = useUserStore();

  // 如果用户访问的不是登录页面，并且未登录，则重定向到登录页面
  if (to.name !== 'login' && !userStore.Token) {
    next({ name: 'login' });
  } else {
    next(); // 放行
  }
});

export default router;